#include <iostream>
#include <memory>
#include <string>

#include <grpc++/grpc++.h>

#include "hashprobe.grpc.pb.h"
#include "types.h"
#include "uci.h"

using grpc::Channel;
using grpc::ClientContext;
using grpc::Status;
using namespace hashprobe;

std::string FormatMove(const HashProbeMove &move) {
  if (move.from_sq().empty()) return "MOVE_NONE";
  return move.from_sq() + move.to_sq() + move.promotion();
}

int main(int argc, char** argv) {
  std::shared_ptr<Channel> channel(grpc::CreateChannel(
      "localhost:50051", grpc::InsecureChannelCredentials()));
  std::unique_ptr<HashProbe::Stub> stub(HashProbe::NewStub(channel));

  for ( ;; ) {
    char buf[256];
    if (fgets(buf, sizeof(buf), stdin) == nullptr || buf[0] == '\n') {
      exit(0);
    }

    char *ptr = strchr(buf, '\n');
    if (ptr != nullptr) *ptr = 0;

    HashProbeRequest request;
    request.set_fen(buf);

    HashProbeResponse response;
    ClientContext context;
    Status status = stub->Probe(&context, request, &response);

    if (status.ok()) {
      for (const HashProbeLine &line : response.line()) {
        std::cout << FormatMove(line.move()) << " ";
        std::cout << line.found() << " ";
	for (const HashProbeMove &move : line.pv()) {
          std::cout << FormatMove(move) << ",";
        }
        std::cout << " ";
        switch (line.bound()) {
        case HashProbeLine::BOUND_NONE:
          std::cout << "?";
          break;
        case HashProbeLine::BOUND_EXACT:
          std::cout << "==";
          break;
        case HashProbeLine::BOUND_UPPER:
          std::cout << "<=";
          break;
        case HashProbeLine::BOUND_LOWER:
          std::cout << ">=";
          break;
        } 
	switch (line.value().score_type()) {
	case HashProbeScore::SCORE_CP:
	  std::cout << " cp " << line.value().score_cp() << " ";
          break;
	case HashProbeScore::SCORE_MATE:
	  std::cout << " mate " << line.value().score_mate() << " ";
          break;
        }
        std::cout << line.depth() << std::endl;
      }
      std::cout << "END" << std::endl;
    } else {
      std::cout << "ERROR" << std::endl;
    }
  }

  return 0;
}
